@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /home/t-ishii/repository/pgpool/main.c,v 1.27 2005/02/02 14:33:38 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/main.c,v 1.2 2005/02/20 04:17:37 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -85,6 +85,8 @@ long int weight_master;	/* normalized weight of master (0-RAND_MAX range) */
 static int stop_sig = SIGTERM;	/* stopping signal default value */
 static int switch_over_sig = SIGUSR1;	/* switch over signal default value */
 
+static int health_check_timer_expired;		/* non 0 if health check timer expired */
+
 /*
 * pgpool main program
 */
@@ -264,6 +266,8 @@ int main(int argc, char **argv)
 	pool_signal(SIGUSR1, failover_handler);
 	pool_signal(SIGUSR2, failover_handler);
 
+	pool_log("pgpool successfully started");
+
 	/*
 	 * This is the main loop
 	 */
@@ -273,6 +277,7 @@ int main(int argc, char **argv)
 		if (!degenerated && pool_config.health_check_period > 0)
 		{
 			int sts;
+			unsigned int sleep_time;
 
 			pool_log("starting health checking");
 
@@ -290,15 +295,19 @@ int main(int argc, char **argv)
 			/*
 			 * do actual health check. trying to connect to the backend
 			 */
+			health_check_timer_expired = 0;
 			sts = health_check();
-			if (sts == -1)
-			{
-				failover_handler(SIGUSR1);		/* master down */
 
-			}
-			else if (sts == -2)
+			if (health_check_timer_expired)
 			{
-				failover_handler(SIGUSR2);		/* secondary down */
+				if (sts == -1)
+				{
+					failover_handler(SIGUSR1);		/* master down */
+				}
+				else if (sts == -2)
+				{
+					failover_handler(SIGUSR2);		/* secondary down */
+				}
 			}
 
 			if (pool_config.health_check_timeout > 0)
@@ -307,7 +316,11 @@ int main(int argc, char **argv)
 				pool_signal(SIGALRM, SIG_IGN);
 			}
 
-			sleep(pool_config.health_check_period);
+			sleep_time = pool_config.health_check_period;
+			while (sleep_time > 0)
+			{
+				sleep_time = sleep(sleep_time);
+			}
 		}
 		else
 		{
@@ -641,7 +654,7 @@ static RETSIGTYPE exit_handler(int sig)
 {
 	int i;
 
-	POOL_SETMASK(&BlockSig);
+	POOL_SETMASK(&AuthBlockSig);
 
 	/*
 	 * this could happend in a child process if a signal has been sent
@@ -654,7 +667,18 @@ static RETSIGTYPE exit_handler(int sig)
 		exit(0);
 	}
 
-	pool_debug("exit_handler called");
+	if (sig == SIGTERM)
+		pool_log("received smart shutdown request");
+	else if (sig == SIGINT)
+		pool_log("received fast shutdown request");
+	else if (sig == SIGQUIT)
+		pool_log("received immediate shutdown request");
+	else
+	{
+		pool_error("exit_handler: unknown signal received %d", sig);
+		POOL_SETMASK(&UnBlockSig);
+		return;
+	}
 
 	exiting = 1;
 
@@ -854,7 +878,7 @@ static RETSIGTYPE failover_handler(int sig)
 static RETSIGTYPE health_check_timer_handler(int sig)
 {
 	POOL_SETMASK(&BlockSig);
-
+	health_check_timer_expired = 1;
 	POOL_SETMASK(&UnBlockSig);
 }
 
